/**
  ******************************************************************************
  * @file    ov7670.c 
  * @author  Wangbao Dev Team
  * @version V1.0.0
  * @date    07-10-2012
  * @brief   uart program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 
#include "ov7670.h"
#include "sccb_bus.h"
#include "lcd.h"
#include "ov7670_table.h"

//#define LCD_BUF 0x6C000002

static void _delay(uint32_t dl)
{
	unsigned int i,y;
	for(i = 0; i < 1000; i++)
	{
		for(y = 0; y < dl; y++);
	}
}

static void port_dma_init(void)
{
	DCMI_InitTypeDef DCMI_InitStructure;
	DMA_InitTypeDef  DMA_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;

	RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_DCMI, ENABLE);//DCMI 
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);//DMA2
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA | RCC_AHB1Periph_GPIOB | 
	RCC_AHB1Periph_GPIOC, ENABLE);//ʹDCMIGPIOʱ
	

	GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_DCMI);//DCMI_HSYNC 
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_DCMI);//DCMI_PIXCLK
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_DCMI);//DCMI_D5 			  
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_DCMI);//DCMI_VSYNC 
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource8, GPIO_AF_DCMI);//DCMI_D6 
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource9, GPIO_AF_DCMI);//DCMI_D7 			  
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource6, GPIO_AF_DCMI);//DCMI_D0 
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource7, GPIO_AF_DCMI);//DCMI_D1 			  
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource8, GPIO_AF_DCMI);//DCMI_D2 
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource9, GPIO_AF_DCMI);//DCMI_D3 
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource11, GPIO_AF_DCMI);//DCMI_D4 

	/* DCMI GPIO configuration ************************************************* */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4 | GPIO_Pin_6;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;  
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7| GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_11;
	GPIO_Init(GPIOC, &GPIO_InitStructure);


	DCMI_InitStructure.DCMI_CaptureMode = DCMI_CaptureMode_Continuous;
	DCMI_InitStructure.DCMI_SynchroMode = DCMI_SynchroMode_Hardware;
	DCMI_InitStructure.DCMI_PCKPolarity = DCMI_PCKPolarity_Rising;
	DCMI_InitStructure.DCMI_VSPolarity = DCMI_VSPolarity_Low;
	DCMI_InitStructure.DCMI_HSPolarity = DCMI_HSPolarity_Low;
	DCMI_InitStructure.DCMI_CaptureRate = DCMI_CaptureRate_All_Frame;
	DCMI_InitStructure.DCMI_ExtendedDataMode = DCMI_ExtendedDataMode_8b;
	DCMI_Init(&DCMI_InitStructure); 
	/* DCMI Interrupts config ***************************************************/
	DCMI_ITConfig(DCMI_IT_VSYNC, ENABLE);	

	
	NVIC_InitStructure.NVIC_IRQChannel = DCMI_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;  
	NVIC_Init(&NVIC_InitStructure); 


	DMA_DeInit(DMA2_Stream1);
	DMA_InitStructure.DMA_Channel = DMA_Channel_1;  
	DMA_InitStructure.DMA_PeripheralBaseAddr = DCMI_DR_ADDRESS;	
	DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)FSMC_LCD_ADDRESS;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralToMemory;
	DMA_InitStructure.DMA_BufferSize = 1;
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Enable;
	DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_Full;
	DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
	DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;  
	DMA_Init(DMA2_Stream1, &DMA_InitStructure);
}

//#if ov2640
// #define R_ID  0x61
// #define W_ID  0x60
////дOV2640Ĵ
//uint8_t ov2640_write_reg(uint8_t reg, uint8_t data)
//{
//start:
//	systick_delay_us(10);
//	sccb_bus_start();//SCCB ߿ʼ
//	if(sccb_bus_write_byte(0x60) == 0)//дַ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 1;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(reg) == 0)//ID
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 2;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(data)==0)//дݵ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 3;//󷵻
//	}
//	sccb_bus_stop();//SCCB ֹͣ	
//	return 0;//ɹ
//}
////OV2640Ĵ
//uint8_t ov2640_read_reg(uint8_t reg, uint8_t *ret_reg)
//{
//start:
//	systick_delay_us(10);
//	//ͨдüĴַ
//	sccb_bus_start();
//	if(sccb_bus_write_byte(0x60) == 0)//дַ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 1;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(reg)==0)//ID
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 2;//󷵻
//	}
//	sccb_bus_stop();//SCCB ֹͣ	
//	systick_delay_us(10);	
//	//üĴַ󣬲Ƕ
//	sccb_bus_start();
//	if(sccb_bus_write_byte(0x61)==0)//ַ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 3;//󷵻
//	}
//	systick_delay_us(10);
//	*ret_reg = sccb_bus_read_byte();//ضֵ
//	sccb_bus_send_noack();//NACK
//	sccb_bus_stop();//SCCB ֹͣ
//	return 0;//ɹ
//}

//void ov2640_reset(void)
//{
//	GPIO_InitTypeDef  GPIO_InitStructure;
//	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);

//	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;  
//	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;			
//	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
//	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
//	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
//	GPIO_Init(GPIOF, &GPIO_InitStructure);

//	GPIO_ResetBits(GPIOF,GPIO_Pin_10);
//	_delay(500);
//	GPIO_SetBits(GPIOF,GPIO_Pin_10);
//	_delay(1000);


//	ov2640_write_reg(0xFF, 0x01);
//	ov2640_write_reg(0x12, 0x80);
//	_delay(1000);
//}
//uint16_t ov2640_get_id(void)
//{
//	uint8_t pidh=0,pidl=0;
//	uint16_t temp=0;
//	ov2640_write_reg(0xff, 0x01);
//	ov2640_read_reg(0x0a,&pidh);
//	ov2640_read_reg(0x0b,&pidl);
//	temp=(uint16_t)(((uint16_t)(pidh<<8)) | pidl);

//	return temp;
//}

//uint16_t ov2640_check(void)
//{
//	uint16_t ovid;
//	port_dma_init();
//	sccb_bus_init();

//	ov2640_reset();
//	systick_delay_ms(5);
//	
//	ovid = ov2640_get_id();
//	return ovid; 
//}
//#elif mt9d111
 #define R_ID  0xbb
 #define W_ID  0xba
//дMT9D111Ĵ

unsigned char mt9d111_write_reg(unsigned char Page,unsigned char regID, unsigned int regDat)
{     
	sccb_bus_start();
	if(0==sccb_bus_write_byte(0xba))
	{
		sccb_bus_stop();
		return(1);
	}
	systick_delay_us(100);
	if(0==sccb_bus_write_byte(0xf0))
	{
		sccb_bus_stop();
		return(1);
	}
	systick_delay_us(100);
	if(0==sccb_bus_write_byte((unsigned char)( Page>>8)))
	{
		sccb_bus_stop();
		return(1);
	}
	if(0==sccb_bus_write_byte((unsigned char)( Page)))
	{
		sccb_bus_stop();
		return(1);
	}
	sccb_bus_stop();
	//////////////////////////////////////////////////////////////     
	sccb_bus_start();
	if(0==sccb_bus_write_byte(0xba))
	{
		sccb_bus_stop();
		return(1);
	}
	systick_delay_us(100);
	if(0==sccb_bus_write_byte(regID))
	{
		sccb_bus_stop();
		return(1);
	}
	systick_delay_us(100);
	if(0==sccb_bus_write_byte((unsigned char)(regDat>>8)))
	{
		sccb_bus_stop();
		return(1);
	}

	if(0==sccb_bus_write_byte((unsigned char)regDat))
	{
		sccb_bus_stop();
		return(1);
	}

	sccb_bus_stop();

	return(0);
}


//uint8_t mt9d111_write_reg(uint8_t Page, uint8_t reg, uint16_t data)
//{
//start:
//	systick_delay_us(10);
//	sccb_bus_start();//SCCB ߿ʼ
//	if(sccb_bus_write_byte(0xba) == 0)//дַ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 1;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(0xf0) == 0)//ID
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 2;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(Page>>8)==0)//дݵ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 3;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(Page)==0)//дݵ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start;
//		return 3;//󷵻
//	}
//	sccb_bus_stop();//SCCB ֹͣ	
//start2:
//	systick_delay_us(10);
//	sccb_bus_start();//SCCB ߿ʼ
//	if(sccb_bus_write_byte(0xba) == 0)//дַ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start2;
//		return 1;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(reg) == 0)//ID
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start2;
//		return 2;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(data>>8)==0)//дݵ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start2;
//		return 3;//󷵻
//	}
//	systick_delay_us(10);
//	if(sccb_bus_write_byte(data)==0)//дݵ
//	{
//		sccb_bus_stop();//SCCB ֹͣ
//		goto start2;
//		return 3;//󷵻
//	}
//	sccb_bus_stop();//SCCB ֹͣ	
//	return 0;//ɹ
//}

//MT9D111Ĵ
uint8_t mt9d111_read_reg(uint8_t Page, uint8_t reg, uint16_t *ret_reg)
{
start:
	systick_delay_us(10);
	//ͨдüĴַ
	sccb_bus_start();
	if(sccb_bus_write_byte(0xba) == 0)//дַ
	{
		sccb_bus_stop();//SCCB ֹͣ
		goto start;
		return 1;//󷵻
	}
	systick_delay_us(10);
	if(sccb_bus_write_byte(0xf0)==0)//ID
	{
		sccb_bus_stop();//SCCB ֹͣ
		goto start;
		return 2;//󷵻
	}
	systick_delay_us(10);
	if(sccb_bus_write_byte(Page>>8)==0)//ID
	{
		sccb_bus_stop();//SCCB ֹͣ
		goto start;
		return 2;//󷵻
	}
	systick_delay_us(10);
	if(sccb_bus_write_byte(Page)==0)//ID
	{
		sccb_bus_stop();//SCCB ֹͣ
		goto start;
		return 2;//󷵻
	}
	sccb_bus_stop();//SCCB ֹͣ	
	systick_delay_us(10);	
	
	//üĴַ󣬲Ƕ
	sccb_bus_start();
	if(sccb_bus_write_byte(0xbb)==0)//ַ
	{
		sccb_bus_stop();//SCCB ֹͣ
		goto start;
		return 3;//󷵻
	}
	systick_delay_us(10);
	
	*ret_reg = sccb_bus_read_byte();//ضֵ
	sccb_bus_send_ack();//ACK
	*ret_reg <<= 8;
	*ret_reg |= sccb_bus_read_byte();
	
	sccb_bus_send_noack();//NACK
	sccb_bus_stop();//SCCB ֹͣ
	return 0;//ɹ
}

void mt9d111_reset(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;  
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;			
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOF, &GPIO_InitStructure);

	GPIO_ResetBits(GPIOF,GPIO_Pin_10);
	_delay(500);
	GPIO_SetBits(GPIOF,GPIO_Pin_10);
	_delay(1000);


	mt9d111_write_reg(0, 0x65, 0xa000);
	mt9d111_write_reg(1, 0xc3, 0x0501);
	mt9d111_write_reg(0, 0x0d, 0x0021);
	mt9d111_write_reg(0, 0x0d, 0x0000);
	_delay(1000);
}
//uint16_t mt9d111_get_id(void)
//{
////	uint8_t pidh=0,pidl=0;
////	uint16_t temp=0;
////	ov2640_write_reg(0xff, 0x01);
////	ov2640_read_reg(0x0a,&pidh);
////	ov2640_read_reg(0x0b,&pidl);
////	temp=(uint16_t)(((uint16_t)(pidh<<8)) | pidl);
//	return (uint16_t)0x90; //temp;
//}

//uint16_t ov2640_check(void)
//{
//	uint16_t ovid;
//	port_dma_init();
//	sccb_bus_init();

//	ov2640_reset();
//	systick_delay_ms(5);
//	
//	ovid = ov2640_get_id();
//	return ovid; 
//}
//#endif

void cam_start(void)
{	
	DMA_Cmd(DMA2_Stream1, ENABLE); 
	DCMI_Cmd(ENABLE); 
	systick_delay_ms(5);
	DCMI_CaptureCmd(ENABLE); 
	//rw_data_prepare();
	lcd_SetCursor(0,0);
	LCD_REG = 0x22;
}

///**
//  * @brief  Configures the OV2640 camera brightness.
//  * @param  Brightness: Brightness value, where Brightness can be: 
//  *         0x40 for Brightness +2,
//  *         0x30 for Brightness +1,
//  *         0x20 for Brightness 0,
//  *         0x10 for Brightness -1,
//  *         0x00 for Brightness -2,
//  * @retval None
//  */
//static void OV2640_BrightnessConfig(uint8_t Brightness)
//{
//  ov2640_write_reg(0xff, 0x00);
//  ov2640_write_reg(0x7c, 0x00);
//  ov2640_write_reg(0x7d, 0x04);
//  ov2640_write_reg(0x7c, 0x09);
//  ov2640_write_reg(0x7d, Brightness);
//  ov2640_write_reg(0x7d, 0x00);
//}

///**
//  * @brief  Configures the OV2640 camera Black and white mode.
//  * @param  BlackWhite: BlackWhite value, where BlackWhite can be: 
//  *         0x18 for B&W,
//  *         0x40 for Negative,
//  *         0x58 for B&W negative,
//  *         0x00 for Normal,
//  * @retval None
//  */
//void OV2640_BandWConfig(uint8_t BlackWhite)
//{
//  ov2640_write_reg(0xff, 0x00);
//  ov2640_write_reg(0x7c, 0x00);
//  ov2640_write_reg(0x7d, BlackWhite);
//  ov2640_write_reg(0x7c, 0x05);
//  ov2640_write_reg(0x7d, 0x80);
//  ov2640_write_reg(0x7d, 0x80);
//}

///**
//  * @brief  Configures the OV2640 camera color effects.
//  * @param  value1: Color effects value1
//  * @param  value2: Color effects value2
//  *         where value1 and value2 can be: 
//  *         value1 = 0x40, value2 = 0xa6 for Antique,
//  *         value1 = 0xa0, value2 = 0x40 for Bluish,
//  *         value1 = 0x40, value2 = 0x40 for Greenish,
//  *         value1 = 0x40, value2 = 0xc0 for Reddish,
//  * @retval None
//  */
//void OV2640_ColorEffectsConfig(uint8_t value1, uint8_t value2)
//{
//  ov2640_write_reg(0xff, 0x00);
//  ov2640_write_reg(0x7c, 0x00);
//  ov2640_write_reg(0x7d, 0x18);
//	ov2640_write_reg(0x7c, 0x05);
//	ov2640_write_reg(0x7d, value1);
//  ov2640_write_reg(0x7d, value2);
//}

///**
//  * @brief  Configures the OV2640 camera contrast.
//  * @param  value1: Contrast value1
//  * @param  value2: Contrast value2
//  *         where value1 and value2 can be: 
//  *         value1 = 0x28, value2 = 0x0c for Contrast +2,
//  *         value1 = 0x24, value2 = 0x16 for Contrast +1,
//  *         value1 = 0x20, value2 = 0x20 for Contrast 0,
//  *         value1 = 0x1c, value2 = 0x2a for Contrast -1,
//  *         value1 = 0x18, value2 = 0x34 for Contrast -2,
//  * @retval None
//  */
//void OV2640_ContrastConfig(uint8_t value1, uint8_t value2)
//{
//  ov2640_write_reg(0xff, 0x00);
//  ov2640_write_reg(0x7c, 0x00);
//  ov2640_write_reg(0x7d, 0x04);
//  ov2640_write_reg(0x7c, 0x07);
//  ov2640_write_reg(0x7d, 0x20);
//  ov2640_write_reg(0x7d, value1);
//  ov2640_write_reg(0x7d, value2);
//  ov2640_write_reg(0x7d, 0x06);
//}

///**
//  * @brief  Configures the OV2640 camera color saturation.
//  * @param  value1: Contrast value1
//  * @param  value2: Contrast value2
//  *         where value1 and value2 can be: 
//  *         value1 = 0x68, value2 = 0x68 for saturation +2,
//  *         value1 = 0x58, value2 = 0x58 for saturation +1,
//  *         value1 = 0x48, value2 = 0x48 for saturation 0,
//  *         value1 = 0x38, value2 = 0x38 for saturation -1,
//  *         value1 = 0x28, value2 = 0x28 for saturation -2,
//  * @retval None
//  */
//void OV2640_SaturationConfig(uint8_t value1, uint8_t value2)
//{
//	ov2640_write_reg(0xff, 0x00);
//  ov2640_write_reg(0x7c, 0x00);
//	ov2640_write_reg(0x7d, 0x02);
//  ov2640_write_reg(0x7c, 0x03);
//	ov2640_write_reg(0x7d, value1);
//  ov2640_write_reg(0x7d, value2);
//}

//#if ov7670
//#elif ov2640
//uint8_t ov2640_init(void)
//{
//	u32 i;
//	ov2640_write_reg(0xff, 0x01);
//  ov2640_write_reg(0x12, 0x80);
//	systick_delay_us(400);;
//	
//	for(i=0;i<(sizeof(OV2640_QVGA))/2;i++)
//	{
//		ov2640_write_reg(OV2640_QVGA[i][0], OV2640_QVGA[i][1]);
//		systick_delay_us(20);
//	}
//	//ƽ
//	OV2640_BrightnessConfig(0x20);
//	/*//ɫͶ*/
//	OV2640_SaturationConfig(0x48,0x48);
//	/*//ջ*/
//	ov2640_write_reg(0xff, 0x00);
//  ov2640_write_reg(0xc7, 0x40);
//	ov2640_write_reg(0xcc, 0x52);
//  ov2640_write_reg(0xcd, 0x41);
//  ov2640_write_reg(0xce, 0x66);
//	
//	//Աȶ
//	OV2640_ContrastConfig(0x20, 0x20);
//	
//	//
//	OV2640_BandWConfig(0x00);
//	
//	return 0;	
//}
//#elif mt9d111


//λ
uint8_t wrMT9D111RegBit(u8 Page, u8 regID, u16 C_Dat,u16 W_Dat)
{
	u16 temp;
	mt9d111_read_reg(Page,regID,&temp);
	temp &=~C_Dat;
	temp |= W_Dat; 
	mt9d111_write_reg(Page,regID,temp);
}
//д
void wrMT9D111Variables(u8 ID,u8 offset, u8 isLogical,	u8 Size, u16 value)
{
	u16 temp;
	temp = (ID<<8)|(offset)|(isLogical<<13)|(Size<<15);
	mt9d111_write_reg(1,198,temp);
	mt9d111_write_reg(1,200,value);
}
//
void rdMT9D111Variables(u8 ID,u8 offset, u8 isLogical,	u8 Size, u16* value)
{
	u16 temp;
	temp = (ID<<8)|(offset)|(isLogical<<13)|(Size<<15);
	mt9d111_write_reg(1,198,temp);
	mt9d111_read_reg(1,200,value);
}

uint8_t mt9d111_init(void)
{
//	unsigned char temp;
//	unsigned int i=0;

	u16  i , temp;  
//	u16 HBLANK_REG,VBLANK_REG; 
	u32  j=0;										 	
//	u8 ovidmsb=0;
//	u8 ovidlsb=0;

	
	port_dma_init();
	sccb_bus_init();
	mt9d111_reset();
	systick_delay_ms(5);
	
//	for(i=0;i<154;i++)//CHANGE_REG_NUM//(sizeof(mt9d111_QVGA))/3
//	{
//		if(mt9d111_QVGA[i][0]>2)
//			{
//			systick_delay_ms(5);//(mt9d111_QVGA[i][2]); // Delay 5ms();
//			}
//		else
//		{
//			if( 0==mt9d111_write_reg(mt9d111_QVGA[i][0],mt9d111_QVGA[i][1],mt9d111_QVGA[i][2]))
//			{
////				return 0;
//			}
//		}
//	//delay_us(100);
//	}
	
	//λSensor
	mt9d111_write_reg(0,0x65,0xA000);
	mt9d111_write_reg(0,0xF0,0x0001);
	mt9d111_write_reg(0,0xC3,0x0501);
	mt9d111_write_reg(0,0xF0,0x0000);
	mt9d111_write_reg(0,0x0D,0x0021);
	systick_delay_ms(1);//ʱ1
	mt9d111_write_reg(0,0x0D,0x0000);
	systick_delay_ms(64);//ʱ64
	
	mt9d111_write_reg(0,0x05,0x01DF);//HBLANK (B) = 479
	mt9d111_write_reg(0,0x06,0x016B);//VBLANK (B) = 363
	mt9d111_write_reg(0,0x07,0x04FF);//HBLANK (A) = 1279
	mt9d111_write_reg(0,0x08,0x000B);//VBLANK (A) = 11
	mt9d111_write_reg(0,0x20,0x0300);//Read Mode (B) = 768
	mt9d111_write_reg(0,0x21,0x8000);//Read Mode (A) = 32768
	mt9d111_write_reg(0,0x66,0x1001);//PLL Control 1 = 4097
	mt9d111_write_reg(0,0x67,0x0501);//PLL Control 2 = 1281
	mt9d111_write_reg(0,0x65,0xA000);//Clock CNTRL: PLL ON = 40960
	mt9d111_write_reg(0,0x65,0x2000);//Clock CNTRL: USE PLL = 8192
	
	//Sequencer States...   
	mt9d111_write_reg(1,0xC6,0xA122);//Enter Preview: Auto Exposure    
	mt9d111_write_reg(1,0xC8,0x0001);//      = 1                       
	mt9d111_write_reg(1,0xC6,0xA123);//Enter Preview: Flicker Detection
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA124);//Enter Preview: Auto White Balance
	mt9d111_write_reg(1,0xC8,0x0001);//      = 1                          
	mt9d111_write_reg(1,0xC6,0xA125);//Enter Preview: Auto Focus        
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA126);//Enter Preview: Histogram         
	mt9d111_write_reg(1,0xC8,0x0001);//      = 1                          
	mt9d111_write_reg(1,0xC6,0xA127);//Enter Preview: Strobe Control    
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA128);//Enter Preview: Skip Control      
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA129);//In Preview: Auto Exposure        
	mt9d111_write_reg(1,0xC8,0x0003);//      = 3                          
	mt9d111_write_reg(1,0xC6,0xA12A);//In Preview: Flicker Detection    
	mt9d111_write_reg(1,0xC8,0x0002);//      = 2                          
	mt9d111_write_reg(1,0xC6,0xA12B);//In Preview: Auto White Balance   
	mt9d111_write_reg(1,0xC8,0x0003);//      = 3                          
	mt9d111_write_reg(1,0xC6,0xA12C);//In Preview: Auto Focus           
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA12D);//In Preview: Histogram            
	mt9d111_write_reg(1,0xC8,0x0003);//      = 3                          
	mt9d111_write_reg(1,0xC6,0xA12E);//In Preview: Strobe Control       
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA12F);//In Preview: Skip Control         
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA130);//Exit Preview: Auto Exposure      
	mt9d111_write_reg(1,0xC8,0x0004);//      = 4                          
	mt9d111_write_reg(1,0xC6,0xA131);//Exit Preview: Flicker Detection  
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA132);//Exit Preview: Auto White Balance 
	mt9d111_write_reg(1,0xC8,0x0001);//      = 1                          
	mt9d111_write_reg(1,0xC6,0xA133);//Exit Preview: Auto Focus         
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA134);//Exit Preview: Histogram          
	mt9d111_write_reg(1,0xC8,0x0001);//      = 1                          
	mt9d111_write_reg(1,0xC6,0xA135);//Exit Preview: Strobe Control     
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA136);//Exit Preview: Skip Control       
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA137);//Capture: Auto Exposure           
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA138);//Capture: Flicker Detection       
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA139);//Capture: Auto White Balance      
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA13A);//Capture: Auto Focus              
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA13B);//Capture: Histogram               
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA13C);//Capture: Strobe Control          
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	mt9d111_write_reg(1,0xC6,0xA13D);//Capture: Skip Control            
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                          
	
	mt9d111_write_reg(1,0xC6,0x2703);//Output Width (A)    
//	mt9d111_write_reg(1,0xC8,0x0320);//      = 800 
	mt9d111_write_reg(1,0xC8,0x0280);//      = 640 
	mt9d111_write_reg(1,0xC6,0x2705);//Output Height (A)
//	mt9d111_write_reg(1,0xC8,0x0258);//      = 600
	
	
	mt9d111_write_reg(1,0xC8,0x01E0);//      = 480                   
	mt9d111_write_reg(1,0xC6,0x2707);//Output Width (B)               
	mt9d111_write_reg(1,0xC8,0x0640);//      = 1600                   
	mt9d111_write_reg(1,0xC6,0x2709);//Output Height (B)              
	mt9d111_write_reg(1,0xC8,0x04B0);//      = 1200                   
	mt9d111_write_reg(1,0xC6,0x270B);//mode_config                    
	mt9d111_write_reg(1,0xC8,0x0030);//      = 48                     
	mt9d111_write_reg(1,0xC6,0x270F);//Row Start (A)                  
	mt9d111_write_reg(1,0xC8,0x001C);//      = 28                     
	mt9d111_write_reg(1,0xC6,0x2711);//Column Start (A)               
	mt9d111_write_reg(1,0xC8,0x003C);//      = 60                     
	mt9d111_write_reg(1,0xC6,0x2713);//Row Height (A)                 
	mt9d111_write_reg(1,0xC8,0x04B0);//      = 1200                   
	mt9d111_write_reg(1,0xC6,0x2715);//Column Width (A)               
	mt9d111_write_reg(1,0xC8,0x0640);//      = 1600                   
	mt9d111_write_reg(1,0xC6,0x2717);//Extra Delay (A)                
	mt9d111_write_reg(1,0xC8,0x022A);//      = 554                    
	mt9d111_write_reg(1,0xC6,0x2719);//Row Speed (A)                  
	mt9d111_write_reg(1,0xC8,0x0011);//      = 17                     
	mt9d111_write_reg(1,0xC6,0x271B);//Row Start (B)                  
	mt9d111_write_reg(1,0xC8,0x001C);//      = 28                     
	mt9d111_write_reg(1,0xC6,0x271D);//Column Start (B)               
	mt9d111_write_reg(1,0xC8,0x003C);//      = 60                     
	mt9d111_write_reg(1,0xC6,0x271F);//Row Height (B)                 
	mt9d111_write_reg(1,0xC8,0x04B0);//      = 1200                   
	mt9d111_write_reg(1,0xC6,0x2721);//Column Width (B)               
	mt9d111_write_reg(1,0xC8,0x0640);//      = 1600                   
	mt9d111_write_reg(1,0xC6,0x2723);//Extra Delay (B)                
	mt9d111_write_reg(1,0xC8,0x000B);//      = 11                     
	mt9d111_write_reg(1,0xC6,0x2725);//Row Speed (B)                  
	mt9d111_write_reg(1,0xC8,0x0011);//      = 17                     
	mt9d111_write_reg(1,0xC6,0x2727);//Crop_X0 (A)                    
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                      
	mt9d111_write_reg(1,0xC6,0x2729);//Crop_X1 (A)                    
	mt9d111_write_reg(1,0xC8,0x0320);//      = 800                    
	mt9d111_write_reg(1,0xC6,0x272B);//Crop_Y0 (A)                    
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                      
	mt9d111_write_reg(1,0xC6,0x272D);//Crop_Y1 (A)                    
	mt9d111_write_reg(1,0xC8,0x0258);//      = 600                    
	mt9d111_write_reg(1,0xC6,0x2735);//Crop_X0 (B)                    
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                      
	mt9d111_write_reg(1,0xC6,0x2737);//Crop_X1 (B)                    
	mt9d111_write_reg(1,0xC8,0x0640);//      = 1600                   
	mt9d111_write_reg(1,0xC6,0x2739);//Crop_Y0 (B)                    
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                      
	mt9d111_write_reg(1,0xC6,0x273B);//Crop_Y1 (B)                    
	mt9d111_write_reg(1,0xC8,0x04B0);//      = 1200                   
	mt9d111_write_reg(1,0xC6,0xA743);//Gamma and Contrast Settings (A)
	mt9d111_write_reg(1,0xC8,0x0042);//      = 66                     
	mt9d111_write_reg(1,0xC6,0xA744);//Gamma and Contrast Settings (B)
	mt9d111_write_reg(1,0xC8,0x0042);//      = 66  


	mt9d111_write_reg(1,0xC6,0xA745);//Gamma Table 0 (A)         
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                 
	mt9d111_write_reg(1,0xC6,0xA746);//Gamma Table 1 (A)         
	mt9d111_write_reg(1,0xC8,0x0014);//      = 20                
	mt9d111_write_reg(1,0xC6,0xA747);//Gamma Table 2 (A)         
	mt9d111_write_reg(1,0xC8,0x0023);//      = 35                
	mt9d111_write_reg(1,0xC6,0xA748);//Gamma Table 3 (A)         
	mt9d111_write_reg(1,0xC8,0x003A);//      = 58                
	mt9d111_write_reg(1,0xC6,0xA749);//Gamma Table 4 (A)         
	mt9d111_write_reg(1,0xC8,0x005E);//      = 94                
	mt9d111_write_reg(1,0xC6,0xA74A);//Gamma Table 5 (A)         
	mt9d111_write_reg(1,0xC8,0x0076);//      = 118               
	mt9d111_write_reg(1,0xC6,0xA74B);//Gamma Table 6 (A)         
	mt9d111_write_reg(1,0xC8,0x0088);//      = 136               
	mt9d111_write_reg(1,0xC6,0xA74C);//Gamma Table 7 (A)         
	mt9d111_write_reg(1,0xC8,0x0096);//      = 150               
	mt9d111_write_reg(1,0xC6,0xA74D);//Gamma Table 8 (A)         
	mt9d111_write_reg(1,0xC8,0x00A3);//      = 163               
	mt9d111_write_reg(1,0xC6,0xA74E);//Gamma Table 9 (A)         
	mt9d111_write_reg(1,0xC8,0x00AF);//      = 175               
	mt9d111_write_reg(1,0xC6,0xA74F);//Gamma Table 10 (A)        
	mt9d111_write_reg(1,0xC8,0x00BA);//      = 186               
	mt9d111_write_reg(1,0xC6,0xA750);//Gamma Table 11 (A)        
	mt9d111_write_reg(1,0xC8,0x00C4);//      = 196               
	mt9d111_write_reg(1,0xC6,0xA751);//Gamma Table 12 (A)        
	mt9d111_write_reg(1,0xC8,0x00CE);//      = 206               
	mt9d111_write_reg(1,0xC6,0xA752);//Gamma Table 13 (A)        
	mt9d111_write_reg(1,0xC8,0x00D7);//      = 215               
	mt9d111_write_reg(1,0xC6,0xA753);//Gamma Table 14 (A)        
	mt9d111_write_reg(1,0xC8,0x00E0);//      = 224               
	mt9d111_write_reg(1,0xC6,0xA754);//Gamma Table 15 (A)        
	mt9d111_write_reg(1,0xC8,0x00E8);//      = 232               
	mt9d111_write_reg(1,0xC6,0xA755);//Gamma Table 16 (A)        
	mt9d111_write_reg(1,0xC8,0x00F0);//      = 240               
	mt9d111_write_reg(1,0xC6,0xA756);//Gamma Table 17 (A)        
	mt9d111_write_reg(1,0xC8,0x00F8);//      = 248               
	mt9d111_write_reg(1,0xC6,0xA757);//Gamma Table 18 (A)        
	mt9d111_write_reg(1,0xC8,0x00FF);//      = 255               
	mt9d111_write_reg(1,0xC6,0xA758);//Gamma Table 0 (B)         
	mt9d111_write_reg(1,0xC8,0x0000);//      = 0                 
	mt9d111_write_reg(1,0xC6,0xA759);//Gamma Table 1 (B)         
	mt9d111_write_reg(1,0xC8,0x0014);//      = 20                
	mt9d111_write_reg(1,0xC6,0xA75A);//Gamma Table 2 (B)         
	mt9d111_write_reg(1,0xC8,0x0023);//      = 35                
	mt9d111_write_reg(1,0xC6,0xA75B);//Gamma Table 3 (B)         
	mt9d111_write_reg(1,0xC8,0x003A);//      = 58                
	mt9d111_write_reg(1,0xC6,0xA75C);//Gamma Table 4 (B)         
	mt9d111_write_reg(1,0xC8,0x005E);//      = 94                
	mt9d111_write_reg(1,0xC6,0xA75D);//Gamma Table 5 (B)         
	mt9d111_write_reg(1,0xC8,0x0076);//      = 118               
	mt9d111_write_reg(1,0xC6,0xA75E);//Gamma Table 6 (B)         
	mt9d111_write_reg(1,0xC8,0x0088);//      = 136               
	mt9d111_write_reg(1,0xC6,0xA75F);//Gamma Table 7 (B)         
	mt9d111_write_reg(1,0xC8,0x0096);//      = 150               
	mt9d111_write_reg(1,0xC6,0xA760);//Gamma Table 8 (B)         
	mt9d111_write_reg(1,0xC8,0x00A3);//      = 163               
	mt9d111_write_reg(1,0xC6,0xA761);//Gamma Table 9 (B)         
	mt9d111_write_reg(1,0xC8,0x00AF);//      = 175               
	mt9d111_write_reg(1,0xC6,0xA762);//Gamma Table 10 (B)        
	mt9d111_write_reg(1,0xC8,0x00BA);//      = 186               
	mt9d111_write_reg(1,0xC6,0xA763);//Gamma Table 11 (B)        
	mt9d111_write_reg(1,0xC8,0x00C4);//      = 196               
	mt9d111_write_reg(1,0xC6,0xA764);//Gamma Table 12 (B)        
	mt9d111_write_reg(1,0xC8,0x00CE);//      = 206               
	mt9d111_write_reg(1,0xC6,0xA765);//Gamma Table 13 (B)        
	mt9d111_write_reg(1,0xC8,0x00D7);//      = 215               
	mt9d111_write_reg(1,0xC6,0xA766);//Gamma Table 14 (B)        
	mt9d111_write_reg(1,0xC8,0x00E0);//      = 224               
	mt9d111_write_reg(1,0xC6,0xA767);//Gamma Table 15 (B)        
	mt9d111_write_reg(1,0xC8,0x00E8);//      = 232               
	mt9d111_write_reg(1,0xC6,0xA768);//Gamma Table 16 (B)        
	mt9d111_write_reg(1,0xC8,0x00F0);//      = 240               
	mt9d111_write_reg(1,0xC6,0xA769);//Gamma Table 17 (B)        
	mt9d111_write_reg(1,0xC8,0x00F8);//      = 248               
	mt9d111_write_reg(1,0xC6,0xA76A);//Gamma Table 18 (B)        
	mt9d111_write_reg(1,0xC8,0x00FF);//      = 255               


	mt9d111_write_reg(1,0xC6,0x276D);//FIFO_Conf1 (A)            
	mt9d111_write_reg(1,0xC8,0xE0E1);//      = 57569             
	mt9d111_write_reg(1,0xC6,0xA76F);//FIFO_Conf2 (A)            
	mt9d111_write_reg(1,0xC8,0x00E1);//      = 225               
	mt9d111_write_reg(1,0xC6,0x2774);//FIFO_Conf1 (B)            
	mt9d111_write_reg(1,0xC8,0xE0E1);//      = 57569             
	mt9d111_write_reg(1,0xC6,0xA776);//FIFO_Conf2 (B)            
	mt9d111_write_reg(1,0xC8,0x00E1);//      = 225               
	mt9d111_write_reg(1,0xC6,0x220B);//Max R12 (B)(Shutter Delay)
	mt9d111_write_reg(1,0xC8,0x0300);//      = 768               
	mt9d111_write_reg(1,0xC6,0xA217);//IndexTH23                 
	mt9d111_write_reg(1,0xC8,0x0008);//      = 8                 
	mt9d111_write_reg(1,0xC6,0x2228);//RowTime (msclk per)/4     
	mt9d111_write_reg(1,0xC8,0x0207);//      = 519               
	mt9d111_write_reg(1,0xC6,0x222F);//R9 Step                   
	mt9d111_write_reg(1,0xC8,0x005F);//      = 95                
	mt9d111_write_reg(1,0xC6,0xA408);//search_f1_50              
	mt9d111_write_reg(1,0xC8,0x0015);//      = 21                
	mt9d111_write_reg(1,0xC6,0xA409);//search_f2_50              
	mt9d111_write_reg(1,0xC8,0x0017);//      = 23                
	mt9d111_write_reg(1,0xC6,0xA40A);//search_f1_60              
	mt9d111_write_reg(1,0xC8,0x0012);//      = 18                
	mt9d111_write_reg(1,0xC6,0xA40B);//search_f2_60              
	mt9d111_write_reg(1,0xC8,0x0014);//      = 20                
	mt9d111_write_reg(1,0xC6,0x2411);//R9_Step_60                
	mt9d111_write_reg(1,0xC8,0x005F);//      = 95                
	mt9d111_write_reg(1,0xC6,0x2413);//R9_Step_50                
	mt9d111_write_reg(1,0xC8,0x0072);//      = 114               
	
	systick_delay_ms(500);//ʱ500
	
//	mt9d111_write_reg(1,0xC6,0xA103);//Refresh Sequencer Mode
//	mt9d111_write_reg(1,0xC8,0x0005);//      = 6  


	//ģʽ
//	mt9d111_write_reg(1,0xC6,0xA120);
//	mt9d111_write_reg(1,0xC8,0x0002);
//	mt9d111_write_reg(1,0xC6,0xA103);
//	mt9d111_write_reg(1,0xC8,0x0002);
//	//Ԥģʽ
	mt9d111_write_reg(1,0xC6,0xA120);
	mt9d111_write_reg(1,0xC8,0x0000);
	mt9d111_write_reg(1,0xC6,0xA103);
	mt9d111_write_reg(1,0xC8,0x0001);
	
	
	/*
		ǳʼ 
		1. Power-up
		2. Hardware reset
		3. Configure and enable PLL
		4. Configure pad slew rate
		5. Configure preview mode
		6. Configure and enable auto focus
		7. Configure capture mode
		8. Perform lock or capture
	*/		
	/*
		3. Configure and enable PLL
		PLL㹫ʽ
		fPLL, fOUT = fPLL, fIN, x M / [2 x (N+1) x (P+1)]

		Frequency  			Equation 			MIN (MHz) 			MAX (MHz)
		fPFD 				fIN/ (N+1) 				2 					13
		fVCO 				fPFD x M 				110 				240
		fOUT 				fVCO / [2 x (P+1)] 		6 					80
		fIN 										6					64

		:Ҫõ20M	
		M=40
		N=1;
		P=3
	*/

//#if 0
	//ʹPLL
	//1. Program PLL frequency settings, R0x66-67:0
//	mt9d111_write_reg(0,0x66,(58<<8)|(1));
//											//Bits 15:8 M
//	wrMT9D111RegBit(0,0x67,0x7f,2);	//Bits 6:0  P	  //1
//	//2. Power up PLL, R0x65:0[14] = 0	 
//	wrMT9D111RegBit(0,0x65,1<<14,0<<14);
	//3. Wait for PLL settling time >150s
//	for(i=0;i<600*72;i++);				
	//4. Turn off PLL bypass, R0x65:2[15] = 0
//	wrMT9D111RegBit(0,0x65,1<<15,0<<15);
//	for(i=0;i<600*72;i++);
//	for(i=0;i<0x1f;i++)
//	wrMT9D111Variables(1,3,1,1,5);
//#endif
//#if 0		
//	/*
//		4. Configure pad slew rate

//		For example, to configure 160x120 LCD RGB preview, program
//		mode.output_width_A=160
//		mode.output_width_B=120
//		mode.out_format_A=0x20
//		seq.cmd=5

//		14
//		0x0E
//		15:0 	0x0501 		Output PCLK1 & PCLK2 Configuration Register
//		3:0 	0x1 		Output clock frequency divisor N1: This 4-bit register contains an integer divisor used to 
//							divide master clock frequency to obtain the frequency of output clock source PCLK1. A 
//							value of 0 in this register has the same effect as 1. 
//		7:5 	0 			PCLK1 slew rate: The value contained in this 3-bit register determines the slew rate of the 
//							output clock when PCLK1 is selected as its source.
//		11:8 	0x5 		Output Clock Frequency Divisor N2: This 4-bit register contains an integer divisor used to 
//							divide master clock frequency to obtain the frequency of output clock source PCLK2. The 
//							output clock switches from PCLK1 to PCLK2 when the output buffer fullness reaches 50%. 
//							A value of 0 in this register has the same effect as 1.
//		12 		0 			Not used.
//		15:13 	0 			PCLK2 slew rate: The value contained in this 3-bit register determines the slew rate of the 
//							output clock when PCLK2 is selected as its source.
//	*/
//	//ǹDOUT, PIXCLK, FRAME_VALID, and LINE_VALIDʱֵ޸
////	#if 0
//	wrMT9D111Variables(7,109,1,0,0x1|(1<<11));
//	wrMT9D111Variables(7,111,1,0,0x21);

//	wrMT9D111Variables(7,116,1,0,0x50a);
//	wrMT9D111Variables(7,118,1,0,0x3);
//	#else
//	rdMT9D111Variables(7,109,1,0,&temp);
//	wrMT9D111Variables(7,109,1,0,temp);
//	rdMT9D111Variables(7,111,1,0,&temp);
//	wrMT9D111Variables(7,111,1,0,temp);

//	rdMT9D111Variables(7,116,1,0,&temp);
//	wrMT9D111Variables(7,116,1,0,temp);
//	rdMT9D111Variables(7,118,1,0,&temp);
//	wrMT9D111Variables(7,118,1,0,temp);
//	for(i=0;i<600*72;i++);
////	#endif

//#endif
//#if 0
//	/*
//		5. Configure previewand Capture mode
//	*/
//	#define x_size	320//320
//	#define y_size  240//240
//	#define master_clock_freq	8000000
//	#define frame_rate			1
//	#define PIXCLK_PERIOD		1
//	#define BORDER				0
//	
//	#define HBLANK(vblank)	//(master_clock_freq/(frame_rate*((x_size+BORDER)+vblank)*PIXCLK_PERIOD)-(y_size+BORDER))
//	#define VBLANK(hblank)	//(master_clock_freq/(frame_rate*((y_size+BORDER)+hblank)*PIXCLK_PERIOD)-(x_size+BORDER))

//	
////#if 0

////// 	{ // С֡
//			mt9d111_write_reg(1, 0xC6, 0xA20E);
// 	    mt9d111_write_reg(1, 0xC8, 0x001);
// 	    mt9d111_write_reg(1, 0xC6, 0xA20D);
// 	    mt9d111_write_reg(1, 0xC8, 0x001);
////// 	}

////	/*
////		Context A
////		R0x21:0[3:2] 	// Row skip level (00 = 2x, 01 = 4x, 10 = 8x, 11 = 16x)
////		R0x21:0[4] 		// Row skip enable
////		R0x21:0[6:5] 	// Column Skip level (00 = 2x, 01 = 4x, 10 = 8x, 11 = 16x)
////		R0x21:0[7] 		// Column skip enable
////	*/

//	wrMT9D111Variables(7,15,1,0,120);
//	wrMT9D111Variables(7,17,1,0,120);

//	wrMT9D111Variables(7,19,1,0,600);
//	wrMT9D111Variables(7,21,1,0,800);
//	mt9d111_write_reg(0,0x07,0x30e);
//	mt9d111_write_reg(0,0x08,10);

////	

//	mt9d111_write_reg(0,0x03,1200);
//	mt9d111_write_reg(0,0x04,1600);


////	
////	wrMT9D111Variables(7,41,1,0,x_size*2);
////	wrMT9D111Variables(7,45,1,0,y_size*2);


////	wrMT9D111Variables(7,3,1,0,x_size);
////	wrMT9D111Variables(7,5,1,0,y_size);   //out size


////	wrMT9D111Variables(1,3,1,1,5); //ˢ
////	wrMT9D111Variables(7,125,1,1,0x20);	
////	rdMT9D111Variables(7,25,1,0,&temp);

////	

////	wrMT9D111Variables(1,3,1,1,5); //ˢ


//////	wrMT9D111RegBit(1,0xc3,1,0);
//////			//skip
//////
//	wrMT9D111RegBit(0,0x21,(3<<2)|(3<<5),(0<<4)|(0<<7));
//	wrMT9D111RegBit(0,0x21,0<<4,1<<4);
//	wrMT9D111RegBit(0,0x21,3<<5,0<<5);
//	wrMT9D111RegBit(0,0x21,0<<7,1<<7);

////	for(j=0;j<0x5f;j++)
////	{
////		wrMT9D111Variables(1,3,1,1,5); //ˢ
////	}
////#else
//	wrMT9D111Variables(7,7,1,0,000);
//	wrMT9D111Variables(7,9,1,0,000);
//	wrMT9D111Variables(1,3,1,1,5); //ˢ
//	wrMT9D111Variables(7,11,1,0,0x30);
////#endif

//	
//#endif

	/*					````````````````````````````````````````````````
		6. Configure and enable auto focus
	*/

	/*
		7. Configure capture mode
	*/

	/*
		8. Perform lock or capture
	*/
	
//	wrMT9D111RegBit(0,0x21,(0<<7)|(0<<4)|(3<<2)|(3<<5),(0<<7)|(0<<4));
//	wrMT9D111Variables(7,25,1,0,(1<<8));
//	for(i=0;i<600*72/3;i++);
//	wrMT9D111Variables(1,3,1,1,1);
//	for(i=0;i<600*72/3;i++);
//	wrMT9D111Reg(0,0x09,0);
//	#if 0//򿪲
//		mt9d111_write_reg(1,0x49,0x0f);	//Test Pattern R/Monochrome Value
//		mt9d111_write_reg(1,0x4A,0xff);	//Test Pattern G/Monochrome Value
//		mt9d111_write_reg(1,0x4B,0xff);	//Test Pattern B Value
//		mt9d111_write_reg(1,0x48,3);	//for test mode	 3 or 5
//	#endif
//	rdMT9D111Reg(0,0x03,&HBLANK_REG);
//	rdMT9D111Reg(0,0x04,&HBLANK_REG);
//	rdMT9D111Variables(7,3,1,0,&HBLANK_REG);
//	rdMT9D111Variables(7,5,1,0,&HBLANK_REG);   //out size
//
//	rdMT9D111Variables(7,41,1,0,&HBLANK_REG);
//	rdMT9D111Variables(7,45,1,0,&HBLANK_REG);
//	wrMT9D111Reg(0,0x1f,(1<<7)|(1<<15)|(0<<8)|(127<<0));
//	USART1_Receive_Byte();
//	JPEG_OutConfigure(JPEG_X_SIZE,JPEG_Y_SIZE,0,JPEG_X_SIZE*JPEG_Y_SIZE);

//	for(i=0;i<(sizeof(mt9d111_QVGA))/2;i++)
//	{
//		mt9d111_write_reg(mt9d111_QVGA[i][0], mt9d111_QVGA[i][1]);
//		systick_delay_us(20);
//	}

//	for(i=0;i<CHANGE_REG_NUM;i++)
//	{
////		if(mt9d111_QVGA[i][0]>2)
////		{
////			//delay_ms(5);// // Delay 5ms();
////			systick_delay_ms(5);//(mt9d111_QVGA[i][2]);
////		}
////		else
////		{
////			if( 0==mt9d111_write_reg(mt9d111_QVGA[i][0],mt9d111_QVGA[i][1],mt9d111_QVGA[i][2]))
////			{
////				return 0;
////			}
////		}
//			 //delay_us(100);
//	}
	

return 0;	
}

/*
	B JPEG
	1.ÿ
	2.ʹJPEG
	3.ΪJPEG
	3.ɫ
	4.marker interval
	5.лB
*/

void JPEG_OutConfigure(u16 width, u16 height,u8 format,int restart_int)
{
//	u16 temp;
//	extern u8 MODE_JPEG;

//	//Changing the Output Resolution
//	wrMT9D111Variables(7,55,1,0,width*2);
//	wrMT9D111Variables(7,59,1,0,height*2);
//	wrMT9D111Variables(7,7,1,0,width);
//	wrMT9D111Variables(7,9,1,0,height);   //out size
////	wrMT9D111Variables(7,126,1,1,0x20);

//	//Selecting Output Data Formats
////	rdMT9D111Variables(9,6,1,1,&temp);
//	wrMT9D111Variables(9,6,1,1,format);

//	//ʹJPEG
//	rdMT9D111Variables(7,11,1,0,&temp);
//	wrMT9D111Variables(7,11,1,0,temp&(~(1<<5)));
//	//wrMT9D111Variables(7,11,1,0,temp|((1<<5)));

//	wrMT9D111Variables(7,114,1,0,0x18);

//	//
//	rdMT9D111Variables(9,7,1,1,&temp);
//	wrMT9D111Variables(9,7,1,1,temp|(1<<4));

//	//JPEG
//	wrMT9D111Variables(9,2,1,0,width); //width
//	wrMT9D111Variables(9,4,1,0,height);
//	wrMT9D111Variables(9,6,1,1,format);//format
//	rdMT9D111Variables(9,7,1,1,&temp);
//	wrMT9D111Variables(9,7,1,1,temp|(1<<4));
//	wrMT9D111Variables(9,8,1,0,restart_int);
//	//лB
//	rdMT9D111Variables(1,0x20,1,1,&temp);
//	wrMT9D111Variables(1,0x20,1,1,temp|2);		
//#if 0	//	still
//	wrMT9D111Variables(1,32,1,1,0);
//	wrMT9D111Variables(1,33,1,1,3);  //num still
////	wrMT9D111Variables(1,0x3,1,1,5);
////	wrMT9D111Variables(1,0x3,1,1,5);
////	wrMT9D111Variables(1,0x3,1,1,5);
////	wrMT9D111Variables(1,0x3,1,1,5);
////	wrMT9D111Variables(1,0x3,1,1,5);
////	wrMT9D111Variables(1,0x3,1,1,5);
////	wrMT9D111Variables(1,32,1,1,2);
//#endif

//	wrMT9D111Variables(1,0x3,1,1,2);   //2

////	MODE_JPEG = 1;

}



